@@ -99,7 +99,12 @@ assert_attributes (GSList     *attrs,
 
   s = g_string_new ("");
   print_attributes (attrs, s);
-  g_assert_cmpstr (s->str, ==, expected);
+  if (strcmp (s->str, expected) != 0)
+    {
+      g_print ("-----\nattribute list mismatch\nexpected:\n%s-----\nreceived:\n%s-----\n",
+               expected, s->str);
+      g_assert_not_reached ();
+    }
   g_string_free (s, TRUE);
 }
 
@@ -298,6 +303,41 @@ test_list_splice (void)
   pango_attr_list_unref (base);
 }
 
+/* Test that empty lists work in pango_attr_list_splice */
+static void
+test_list_splice2 (void)
+{
+  PangoAttrList *list;
+  PangoAttrList *other;
+  PangoAttribute *attr;
+
+  list = pango_attr_list_new ();
+  other = pango_attr_list_new ();
+
+  pango_attr_list_splice (list, other, 11, 5);
+
+  g_assert_null (pango_attr_list_get_attributes (list));
+
+  attr = pango_attr_size_new (10);
+  attr->start_index = 0;
+  attr->end_index = -1;
+  pango_attr_list_insert (other, attr);
+
+  pango_attr_list_splice (list, other, 11, 5);
+
+  assert_attr_list (list, "[11,-1]size=10\n");
+
+  pango_attr_list_unref (other);
+  other = pango_attr_list_new ();
+
+  pango_attr_list_splice (list, other, 11, 5);
+
+  assert_attr_list (list, "[11,-1]size=10\n");
+
+  pango_attr_list_unref (other);
+  pango_attr_list_unref (list);
+}
+
 static gboolean
 never_true (PangoAttribute *attribute, gpointer user_data)
 {
@@ -618,6 +658,20 @@ test_list_update (void)
   pango_attr_list_unref (list);
 }
 
+/* Test that empty lists work in pango_attr_list_update */
+static void
+test_list_update2 (void)
+{
+  PangoAttrList *list;
+
+  list = pango_attr_list_new ();
+  pango_attr_list_update (list, 8, 10, 20);
+
+  g_assert_null (pango_attr_list_get_attributes (list));
+
+  pango_attr_list_unref (list);
+}
+
 static void
 test_list_equal (void)
 {
@@ -830,6 +884,66 @@ test_merge (void)
   pango_attr_list_unref (list2);
 }
 
+/* reproduce what the links example in gtk4-demo does
+ * with the colored Google link
+ */
+static void
+test_merge2 (void)
+{
+  PangoAttrList *list;
+  PangoAttribute *attr;
+
+  list = pango_attr_list_new ();
+  attr = pango_attr_underline_new (PANGO_UNDERLINE_SINGLE);
+  attr->start_index = 0;
+  attr->end_index = 10;
+  pango_attr_list_insert (list, attr);
+  attr = pango_attr_foreground_new (0, 0, 0xffff);
+  attr->start_index = 0;
+  attr->end_index = 10;
+  pango_attr_list_insert (list, attr);
+
+  assert_attr_list (list, "[0,10]underline=1\n"
+                          "[0,10]foreground=#00000000ffff\n");
+
+  attr = pango_attr_foreground_new (0xffff, 0, 0);
+  attr->start_index = 2;
+  attr->end_index = 3;
+
+  pango_attr_list_change (list, attr);
+
+  assert_attr_list (list, "[0,10]underline=1\n"
+                          "[0,2]foreground=#00000000ffff\n"
+                          "[2,3]foreground=#ffff00000000\n"
+                          "[3,10]foreground=#00000000ffff\n");
+
+  attr = pango_attr_foreground_new (0, 0xffff, 0);
+  attr->start_index = 3;
+  attr->end_index = 4;
+
+  pango_attr_list_change (list, attr);
+
+  assert_attr_list (list, "[0,10]underline=1\n"
+                          "[0,2]foreground=#00000000ffff\n"
+                          "[2,3]foreground=#ffff00000000\n"
+                          "[3,4]foreground=#0000ffff0000\n"
+                          "[4,10]foreground=#00000000ffff\n");
+
+  attr = pango_attr_foreground_new (0, 0, 0xffff);
+  attr->start_index = 4;
+  attr->end_index = 5;
+
+  pango_attr_list_change (list, attr);
+
+  assert_attr_list (list, "[0,10]underline=1\n"
+                          "[0,2]foreground=#00000000ffff\n"
+                          "[2,3]foreground=#ffff00000000\n"
+                          "[3,4]foreground=#0000ffff0000\n"
+                          "[4,10]foreground=#00000000ffff\n");
+
+  pango_attr_list_unref (list);
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -840,11 +954,14 @@ main (int argc, char *argv[])
   g_test_add_func ("/attributes/list/basic", test_list);
   g_test_add_func ("/attributes/list/change", test_list_change);
   g_test_add_func ("/attributes/list/splice", test_list_splice);
+  g_test_add_func ("/attributes/list/splice2", test_list_splice2);
   g_test_add_func ("/attributes/list/filter", test_list_filter);
   g_test_add_func ("/attributes/list/update", test_list_update);
+  g_test_add_func ("/attributes/list/update2", test_list_update2);
   g_test_add_func ("/attributes/list/equal", test_list_equal);
   g_test_add_func ("/attributes/list/insert", test_insert);
   g_test_add_func ("/attributes/list/merge", test_merge);
+  g_test_add_func ("/attributes/list/merge2", test_merge2);
   g_test_add_func ("/attributes/iter/basic", test_iter);
   g_test_add_func ("/attributes/iter/get", test_iter_get);
   g_test_add_func ("/attributes/iter/get_font", test_iter_get_font);
